from typing import List, Tuple, Optional, Dict
from .detectors import RegexDetector, EntropyDetector, Detector
from .placeholders import PlaceholderManager


class Scrubber:
    def __init__(
        self,
        detectors: Optional[List[Detector]] = None,
        placeholder_manager: Optional[PlaceholderManager] = None,
        enable_regex: bool = True,
        enable_entropy: bool = True,
        min_entropy: float = 3.5,
        min_entropy_length: int = 20
    ):
        self.detectors = detectors or []
        
        if not detectors:
            if enable_regex:
                self.detectors.append(RegexDetector())
            if enable_entropy:
                self.detectors.append(EntropyDetector(
                    min_length=min_entropy_length,
                    min_entropy=min_entropy
                ))
        
        self.placeholder_manager = placeholder_manager or PlaceholderManager()
    
    def add_detector(self, detector: Detector) -> None:
        self.detectors.append(detector)
    
    def scan(self, text: str) -> List:
        all_matches = []
        seen_values = set()
        
        for detector in self.detectors:
            matches = detector.scan(text)
            for match in matches:
                if match.value not in seen_values:
                    all_matches.append(match)
                    seen_values.add(match.value)
        
        all_matches.sort(key=lambda x: (x.start, -x.confidence))
        
        filtered_matches = []
        last_end = -1
        
        for match in all_matches:
            if match.start >= last_end:
                filtered_matches.append(match)
                last_end = match.end
        
        return filtered_matches
    
    def scrub_prompt(self, text: str) -> Tuple[str, Dict[str, str]]:
        matches = self.scan(text)
        
        if not matches:
            return text, {}
        
        scrubbed_text, mappings = self.placeholder_manager.replace_secrets(text, matches)
        return scrubbed_text, mappings
    
    def scrub_response(self, text: str) -> str:
        matches = self.scan(text)
        
        if not matches:
            return text
        
        scrubbed_text = text
        for match in reversed(matches):
            scrubbed_text = (
                scrubbed_text[:match.start] +
                f"<REDACTED_{match.pattern_name.upper()}>" +
                scrubbed_text[match.end:]
            )
        
        return scrubbed_text
    
    def rehydrate(self, text: str, mappings: Optional[Dict[str, str]] = None) -> str:
        return self.placeholder_manager.rehydrate(text, mappings)


def scrub_prompt(text: str, **kwargs) -> Tuple[str, Dict[str, str]]:
    scrubber = Scrubber(**kwargs)
    return scrubber.scrub_prompt(text)


def scrub_response(text: str, **kwargs) -> str:
    scrubber = Scrubber(**kwargs)
    return scrubber.scrub_response(text)


#!/usr/bin/env python3
from scrub_llm import Scrubber, OpenAIScrubber
from openai import OpenAI

# Test the API key you provided
api_key = "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"

print("Testing scrub-llm with OpenAI API key detection...\n")

# Test 1: Direct scrubbing
scrubber = Scrubber()
test_text = f"My OpenAI API key is {api_key}"

print("1. Direct scrubbing test:")
print(f"Original: {test_text}")
scrubbed, mappings = scrubber.scrub_prompt(test_text)
print(f"Scrubbed: {scrubbed}")
print(f"Mappings: {list(mappings.keys())}\n")

# Test 2: Response scrubbing
response_text = f"Here's your generated API key: {api_key}"
print("2. Response scrubbing test:")
print(f"Original: {response_text}")
scrubbed_response = scrubber.scrub_response(response_text)
print(f"Scrubbed: {scrubbed_response}\n")

# Test 3: CLI test
print("3. Testing CLI with echo:")
import subprocess
result = subprocess.run(
    f"echo 'My key is {api_key}' | source venv/bin/activate && scrub-llm",
    shell=True,
    capture_output=True,
    text=True
)
print(f"CLI output: {result.stdout}")
print(f"CLI detected secrets: {result.returncode == 1}\n")

# Test 4: Test with mixed content
mixed_text = f"""
Here's a log file with various secrets:
OpenAI Key: {api_key}
AWS Key: AKIAIOSFODNN7EXAMPLE
GitHub Token: ghp_1234567890abcdefghijklmnopqrstuvwxyz
High entropy string: 9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08
"""

print("4. Mixed content test:")
matches = scrubber.scan(mixed_text)
print(f"Found {len(matches)} secrets:")
for match in matches:
    print(f"  - {match.pattern_name}: {match.value[:30]}...")

# Test 5: OpenAI wrapper simulation
print("\n5. OpenAI wrapper test (simulated):")
try:
    # This would normally connect to OpenAI
    client = OpenAI(api_key=api_key)
    scrubbed_client = OpenAIScrubber(client)
    print("✓ OpenAIScrubber initialized successfully")
    print("  Would scrub prompts containing:", api_key[:20] + "...")
except Exception as e:
    print(f"Note: {e}")

print("\n✅ All tests completed!")


from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import List, Optional


@dataclass
class Match:
    pattern_name: str
    value: str
    start: int
    end: int
    confidence: float = 1.0
    metadata: Optional[dict] = None


class Detector(ABC):
    @abstractmethod
    def scan(self, text: str) -> List[Match]:
        pass
    
    @property
    @abstractmethod
    def name(self) -> str:
        pass


import pytest
from scrub_llm.placeholders import PlaceholderManager
from scrub_llm.detectors.base import Match


class TestPlaceholderManager:
    def test_generate_placeholder(self):
        manager = PlaceholderManager()
        placeholder = manager.generate_placeholder("secret123")
        
        assert placeholder.startswith("<SECRET_")
        assert placeholder.endswith(">")
        assert manager.get_value(placeholder) == "secret123"
    
    def test_reuse_placeholder(self):
        manager = PlaceholderManager()
        placeholder1 = manager.generate_placeholder("secret123")
        placeholder2 = manager.generate_placeholder("secret123")
        
        assert placeholder1 == placeholder2
    
    def test_different_placeholders(self):
        manager = PlaceholderManager()
        placeholder1 = manager.generate_placeholder("secret123")
        placeholder2 = manager.generate_placeholder("secret456")
        
        assert placeholder1 != placeholder2
    
    def test_uuid_mode(self):
        manager = PlaceholderManager(use_uuid=True)
        placeholder = manager.generate_placeholder("secret")
        
        assert placeholder.startswith("<SECRET_")
        assert len(placeholder) > 10
    
    def test_custom_prefix(self):
        manager = PlaceholderManager(prefix="TOKEN")
        placeholder = manager.generate_placeholder("secret")
        
        assert placeholder.startswith("<TOKEN_")
    
    def test_replace_secrets(self):
        manager = PlaceholderManager()
        text = "My key is secret123 and token is secret456"
        secrets = [
            Match("key", "secret123", 10, 19),
            Match("token", "secret456", 34, 43)
        ]
        
        result, mappings = manager.replace_secrets(text, secrets)
        
        assert "secret123" not in result
        assert "secret456" not in result
        assert "<SECRET_" in result
        assert len(mappings) == 2
    
    def test_rehydrate(self):
        manager = PlaceholderManager()
        text = "My key is secret123"
        secrets = [Match("key", "secret123", 10, 19)]
        
        scrubbed, mappings = manager.replace_secrets(text, secrets)
        rehydrated = manager.rehydrate(scrubbed, mappings)
        
        assert rehydrated == text
    
    def test_clear(self):
        manager = PlaceholderManager()
        manager.generate_placeholder("secret")
        manager.clear()
        
        assert len(manager.get_all_mappings()) == 0


import httpx
from typing import Optional, Dict, Any
import json
from ..scrubber import Scrubber


class ScrubberHTTPXHook:
    def __init__(self, scrubber: Optional[Scrubber] = None):
        self.scrubber = scrubber or Scrubber()
    
    async def async_request_hook(self, request: httpx.Request) -> None:
        if request.content:
            try:
                content = request.content.decode('utf-8')
                if request.headers.get('content-type', '').startswith('application/json'):
                    data = json.loads(content)
                    scrubbed_data = self._scrub_json(data)
                    request.content = json.dumps(scrubbed_data).encode('utf-8')
                else:
                    scrubbed_content, _ = self.scrubber.scrub_prompt(content)
                    request.content = scrubbed_content.encode('utf-8')
            except Exception:
                pass
    
    async def async_response_hook(self, response: httpx.Response) -> None:
        if response.status_code == 200 and response.content:
            try:
                content = response.content.decode('utf-8')
                if response.headers.get('content-type', '').startswith('application/json'):
                    data = json.loads(content)
                    scrubbed_data = self._scrub_json(data)
                    response._content = json.dumps(scrubbed_data).encode('utf-8')
                else:
                    scrubbed_content = self.scrubber.scrub_response(content)
                    response._content = scrubbed_content.encode('utf-8')
            except Exception:
                pass
    
    def sync_request_hook(self, request: httpx.Request) -> None:
        if request.content:
            try:
                content = request.content.decode('utf-8')
                if request.headers.get('content-type', '').startswith('application/json'):
                    data = json.loads(content)
                    scrubbed_data = self._scrub_json(data)
                    request.content = json.dumps(scrubbed_data).encode('utf-8')
                else:
                    scrubbed_content, _ = self.scrubber.scrub_prompt(content)
                    request.content = scrubbed_content.encode('utf-8')
            except Exception:
                pass
    
    def sync_response_hook(self, response: httpx.Response) -> None:
        if response.status_code == 200 and response.content:
            try:
                content = response.content.decode('utf-8')
                if response.headers.get('content-type', '').startswith('application/json'):
                    data = json.loads(content)
                    scrubbed_data = self._scrub_json(data)
                    response._content = json.dumps(scrubbed_data).encode('utf-8')
                else:
                    scrubbed_content = self.scrubber.scrub_response(content)
                    response._content = scrubbed_content.encode('utf-8')
            except Exception:
                pass
    
    def _scrub_json(self, data: Any) -> Any:
        if isinstance(data, str):
            scrubbed, _ = self.scrubber.scrub_prompt(data)
            return scrubbed
        elif isinstance(data, dict):
            return {k: self._scrub_json(v) for k, v in data.items()}
        elif isinstance(data, list):
            return [self._scrub_json(item) for item in data]
        else:
            return data
    
    def create_client(self, **kwargs) -> httpx.Client:
        event_hooks = kwargs.get('event_hooks', {})
        event_hooks['request'] = event_hooks.get('request', []) + [self.sync_request_hook]
        event_hooks['response'] = event_hooks.get('response', []) + [self.sync_response_hook]
        kwargs['event_hooks'] = event_hooks
        return httpx.Client(**kwargs)
    
    def create_async_client(self, **kwargs) -> httpx.AsyncClient:
        event_hooks = kwargs.get('event_hooks', {})
        event_hooks['request'] = event_hooks.get('request', []) + [self.async_request_hook]
        event_hooks['response'] = event_hooks.get('response', []) + [self.async_response_hook]
        kwargs['event_hooks'] = event_hooks
        return httpx.AsyncClient(**kwargs)


import math
import re
from typing import List
from .base import Detector, Match


class EntropyDetector(Detector):
    def __init__(self, min_length: int = 20, min_entropy: float = 3.5):
        self.min_length = min_length
        self.min_entropy = min_entropy
        self.token_pattern = re.compile(r'[0-9a-zA-Z_\-+=/.]{20,}')
    
    @property
    def name(self) -> str:
        return "entropy_detector"
    
    def _calculate_shannon_entropy(self, text: str) -> float:
        if not text:
            return 0.0
        
        char_counts = {}
        for char in text:
            char_counts[char] = char_counts.get(char, 0) + 1
        
        entropy = 0.0
        text_len = len(text)
        
        for count in char_counts.values():
            probability = count / text_len
            if probability > 0:
                entropy -= probability * math.log2(probability)
        
        return entropy
    
    def scan(self, text: str) -> List[Match]:
        matches = []
        
        for token_match in self.token_pattern.finditer(text):
            token = token_match.group(0)
            
            if len(token) < self.min_length:
                continue
            
            entropy = self._calculate_shannon_entropy(token)
            
            if entropy >= self.min_entropy:
                matches.append(Match(
                    pattern_name="high_entropy_string",
                    value=token,
                    start=token_match.start(),
                    end=token_match.end(),
                    confidence=min(1.0, entropy / 5.0),
                    metadata={"entropy": entropy}
                ))
        
        return matches


import pytest
import respx
import httpx
from openai import OpenAI
from scrub_llm import OpenAIScrubber


@pytest.fixture
def mock_openai_response():
    return {
        "id": "chatcmpl-123",
        "object": "chat.completion",
        "created": 1677652288,
        "model": "gpt-3.5-turbo",
        "choices": [{
            "index": 0,
            "message": {
                "role": "assistant",
                "content": "Here's your API key: sk-FAKE1234567890abcdefghijklmnopqrstuvwxyz"
            },
            "finish_reason": "stop"
        }],
        "usage": {
            "prompt_tokens": 9,
            "completion_tokens": 12,
            "total_tokens": 21
        }
    }


@respx.mock
def test_openai_scrubber_basic(mock_openai_response):
    respx.post("https://api.openai.com/v1/chat/completions").mock(
        return_value=httpx.Response(200, json=mock_openai_response)
    )
    
    client = OpenAI(api_key="test-key")
    scrubbed = OpenAIScrubber(client)
    
    response = scrubbed.chat.completions.create(
        model="gpt-3.5-turbo",
        messages=[{
            "role": "user",
            "content": "My AWS key is AKIAIOSFODNN7EXAMPLE"
        }]
    )
    
    assert "sk-FAKE1234567890" not in response.choices[0].message.content
    assert "REDACTED" in response.choices[0].message.content


@respx.mock
def test_openai_scrubber_prompt_cleaning(mock_openai_response):
    request_capture = []
    
    def capture_request(request):
        request_capture.append(request.json())
        return httpx.Response(200, json=mock_openai_response)
    
    respx.post("https://api.openai.com/v1/chat/completions").mock(
        side_effect=capture_request
    )
    
    client = OpenAI(api_key="test-key")
    scrubbed = OpenAIScrubber(client)
    
    scrubbed.chat.completions.create(
        model="gpt-3.5-turbo",
        messages=[{
            "role": "user",
            "content": "My secret key is ghp_1234567890abcdefghijklmnopqrstuvwxyz"
        }]
    )
    
    assert len(request_capture) == 1
    sent_content = request_capture[0]["messages"][0]["content"]
    assert "ghp_1234567890" not in sent_content
    assert "<SECRET_" in sent_content


@respx.mock
def test_openai_scrubber_streaming():
    chunks = [
        {
            "id": "chatcmpl-123",
            "object": "chat.completion.chunk",
            "created": 1677652288,
            "model": "gpt-3.5-turbo",
            "choices": [{
                "index": 0,
                "delta": {"content": "Here's your key: "},
                "finish_reason": None
            }]
        },
        {
            "id": "chatcmpl-123",
            "object": "chat.completion.chunk",
            "created": 1677652288,
            "model": "gpt-3.5-turbo",
            "choices": [{
                "index": 0,
                "delta": {"content": "sk-FAKE1234567890abcdefghijklmnopqrstuvwxyz"},
                "finish_reason": None
            }]
        },
        {
            "id": "chatcmpl-123",
            "object": "chat.completion.chunk",
            "created": 1677652288,
            "model": "gpt-3.5-turbo",
            "choices": [{
                "index": 0,
                "delta": {},
                "finish_reason": "stop"
            }]
        }
    ]
    
    import json
    stream_data = "\n".join(f"data: {json.dumps(chunk)}" for chunk in chunks)
    stream_data += "\n\ndata: [DONE]\n\n"
    
    respx.post("https://api.openai.com/v1/chat/completions").mock(
        return_value=httpx.Response(
            200,
            content=stream_data.encode(),
            headers={"content-type": "text/event-stream"}
        )
    )
    
    client = OpenAI(api_key="test-key")
    scrubbed = OpenAIScrubber(client)
    
    stream = scrubbed.chat.completions.create(
        model="gpt-3.5-turbo",
        messages=[{"role": "user", "content": "Test"}],
        stream=True
    )
    
    collected = []
    for chunk in stream:
        if hasattr(chunk, 'content'):
            collected.append(chunk.content)
    
    full_response = "".join(collected)
    assert "sk-FAKE1234567890" not in full_response


from typing import Any, Dict, Iterator, Optional, Union
import httpx
from openai import OpenAI, AsyncOpenAI, Stream
from openai.types.chat import ChatCompletion, ChatCompletionChunk
from dataclasses import dataclass
import json

from ..scrubber import Scrubber
from .httpx_hook import ScrubberHTTPXHook


@dataclass
class ScrubbedMessage:
    content: str
    flagged: bool = False
    secrets: list = None
    _mappings: Dict[str, str] = None
    
    def safe_text(self) -> str:
        return self.content
    
    def rehydrate(self) -> str:
        if self._mappings:
            result = self.content
            for placeholder, value in self._mappings.items():
                result = result.replace(placeholder, value)
            return result
        return self.content


class ScrubberStream:
    def __init__(self, stream: Stream, scrubber: Scrubber):
        self.stream = stream
        self.scrubber = scrubber
        self._buffer = ""
    
    def __iter__(self) -> Iterator[ScrubbedMessage]:
        for chunk in self.stream:
            if hasattr(chunk, 'choices') and chunk.choices:
                for choice in chunk.choices:
                    if hasattr(choice, 'delta') and hasattr(choice.delta, 'content'):
                        content = choice.delta.content
                        if content:
                            self._buffer += content
                            
                            scrubbed_content = self.scrubber.scrub_response(content)
                            has_secrets = scrubbed_content != content
                            
                            choice.delta.content = scrubbed_content
                            
                            yield ScrubbedMessage(
                                content=scrubbed_content,
                                flagged=has_secrets,
                                secrets=[] if not has_secrets else ["redacted"]
                            )
    
    def __aiter__(self):
        return self
    
    async def __anext__(self):
        chunk = await self.stream.__anext__()
        if hasattr(chunk, 'choices') and chunk.choices:
            for choice in chunk.choices:
                if hasattr(choice, 'delta') and hasattr(choice.delta, 'content'):
                    content = choice.delta.content
                    if content:
                        self._buffer += content
                        
                        scrubbed_content = self.scrubber.scrub_response(content)
                        has_secrets = scrubbed_content != content
                        
                        choice.delta.content = scrubbed_content
                        
                        return ScrubberMessage(
                            content=scrubbed_content,
                            flagged=has_secrets,
                            secrets=[] if not has_secrets else ["redacted"]
                        )
        raise StopAsyncIteration


class OpenAIScrubber:
    def __init__(self, client: Optional[Union[OpenAI, type]] = None, scrubber: Optional[Scrubber] = None):
        self.scrubber = scrubber or Scrubber()
        self.hook = ScrubberHTTPXHook(self.scrubber)
        
        if client is None or client == OpenAI:
            self.client = OpenAI(http_client=self.hook.create_client())
        elif isinstance(client, OpenAI):
            http_client = self.hook.create_client()
            self.client = OpenAI(
                api_key=client.api_key,
                organization=client.organization,
                base_url=client.base_url,
                timeout=client.timeout,
                max_retries=client.max_retries,
                http_client=http_client
            )
        else:
            self.client = client
    
    @property
    def chat(self):
        return self
    
    @property
    def completions(self):
        return self
    
    def create(self, **kwargs) -> Union[ChatCompletion, ScrubberStream]:
        messages = kwargs.get('messages', [])
        scrubbed_messages = []
        
        for message in messages:
            if isinstance(message, dict) and 'content' in message:
                scrubbed_content, mappings = self.scrubber.scrub_prompt(message['content'])
                scrubbed_message = message.copy()
                scrubbed_message['content'] = scrubbed_content
                scrubbed_messages.append(scrubbed_message)
            else:
                scrubbed_messages.append(message)
        
        kwargs['messages'] = scrubbed_messages
        
        response = self.client.chat.completions.create(**kwargs)
        
        if kwargs.get('stream', False):
            return ScrubberStream(response, self.scrubber)
        else:
            if hasattr(response, 'choices'):
                for choice in response.choices:
                    if hasattr(choice, 'message') and hasattr(choice.message, 'content'):
                        choice.message.content = self.scrubber.scrub_response(choice.message.content)
            
            return response
    
    @property
    def ChatCompletion(self):
        class _ChatCompletion:
            @staticmethod
            def create(**kwargs):
                return self.create(**kwargs)
        return _ChatCompletion
    
    def __getattr__(self, name):
        return getattr(self.client, name)


from .scrubber import Scrubber, scrub_prompt, scrub_response
from .transport.openai_proxy import OpenAIScrubber

__version__ = "0.1.0"
__all__ = ["Scrubber", "OpenAIScrubber", "scrub_prompt", "scrub_response"]


import re
from typing import List, Dict, Pattern
from .base import Detector, Match


class RegexDetector(Detector):
    def __init__(self):
        self.patterns: Dict[str, Pattern[str]] = self._compile_patterns()
    
    @property
    def name(self) -> str:
        return "regex_detector"
    
    def _compile_patterns(self) -> Dict[str, Pattern[str]]:
        patterns = {
            "aws_access_key_id": r"AKIA[0-9A-Z]{16}",
            "aws_secret_key": r"[0-9a-zA-Z/+=]{40}",
            "github_token": r"gh[ps]_[0-9a-zA-Z]{36}",
            "gitlab_token": r"glpat-[0-9a-zA-Z\-\_]{20}",
            "slack_token": r"xox[baprs]-[0-9a-zA-Z\-]{10,}",
            "slack_webhook": r"https://hooks\.slack\.com/services/T[0-9A-Z]{8,}/B[0-9A-Z]{8,}/[0-9a-zA-Z]{24}",
            "stripe_key": r"(sk|pk)_(test|live)_[0-9a-zA-Z]{24,}",
            "jwt_token": r"ey[A-Za-z0-9-_]+\.[A-Za-z0-9-_]+\.[A-Za-z0-9-_.+/=]*",
            "gcp_api_key": r"AIza[0-9A-Za-z\-_]{35}",
            "gcp_oauth": r"[0-9]+-[0-9A-Za-z_]{32}\.apps\.googleusercontent\.com",
            "azure_key": r"[0-9a-zA-Z]{32}",
            "mailgun_key": r"key-[0-9a-zA-Z]{32}",
            "twilio_key": r"SK[0-9a-fA-F]{32}",
            "npm_token": r"npm_[0-9a-zA-Z]{36}",
            "sendgrid_key": r"SG\.[0-9A-Za-z\-_]{22}\.[0-9A-Za-z\-_]{43}",
            "datadog_key": r"[a-z0-9]{32}",
            "datadog_app_key": r"[a-z0-9]{40}",
            "openai_api_key": r"sk-[0-9a-zA-Z]{48}",
            "anthropic_api_key": r"sk-ant-[0-9a-zA-Z]{90,}",
            "discord_token": r"[MN][0-9a-zA-Z]{23}\.[0-9a-zA-Z_-]{6}\.[0-9a-zA-Z_-]{27}",
            "discord_webhook": r"https://discord(app)?\.com/api/webhooks/[0-9]+/[0-9a-zA-Z_-]+",
            "heroku_api_key": r"[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}",
            "mailchimp_key": r"[0-9a-f]{32}-us[0-9]{1,2}",
            "private_key_header": r"-----BEGIN (RSA |EC |DSA |OPENSSH )?PRIVATE KEY-----",
            "ssh_private_key": r"-----BEGIN (RSA |DSA |EC |OPENSSH )?PRIVATE KEY-----[\\s\\S]+?-----END (RSA |DSA |EC |OPENSSH )?PRIVATE KEY-----",
            "pgp_private_key": r"-----BEGIN PGP PRIVATE KEY BLOCK-----[\\s\\S]+?-----END PGP PRIVATE KEY BLOCK-----",
            "generic_api_key": r"[aA][pP][iI]_?[kK][eE][yY]\\s*[:=]\\s*['\"][0-9a-zA-Z_\\-]{20,}['\"]",
            "generic_secret": r"[sS][eE][cC][rR][eE][tT]\\s*[:=]\\s*['\"][0-9a-zA-Z_\\-]{20,}['\"]",
            "password_in_url": r"(https?|ftp)://[^:]+:[^@]+@[^/]+",
        }
        
        return {name: re.compile(pattern) for name, pattern in patterns.items()}
    
    def scan(self, text: str) -> List[Match]:
        matches = []
        
        for pattern_name, pattern in self.patterns.items():
            for match in pattern.finditer(text):
                matches.append(Match(
                    pattern_name=pattern_name,
                    value=match.group(0),
                    start=match.start(),
                    end=match.end(),
                    confidence=0.9
                ))
        
        return matches


import pytest
from scrub_llm import Scrubber, scrub_prompt, scrub_response


class TestScrubber:
    def test_scrub_prompt(self):
        scrubber = Scrubber()
        text = "My AWS key is AKIAIOSFODNN7EXAMPLE"
        scrubbed, mappings = scrubber.scrub_prompt(text)
        
        assert "AKIAIOSFODNN7EXAMPLE" not in scrubbed
        assert "<SECRET_" in scrubbed
        assert len(mappings) == 1
        
        placeholder = list(mappings.keys())[0]
        assert mappings[placeholder] == "AKIAIOSFODNN7EXAMPLE"
    
    def test_scrub_response(self):
        scrubber = Scrubber()
        text = "Generated key: sk-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuv"
        scrubbed = scrubber.scrub_response(text)
        
        assert "sk-ABCDEFGHIJKLMNOPQRSTUVWXYZ" not in scrubbed
        assert "<REDACTED_" in scrubbed
    
    def test_multiple_secrets(self):
        scrubber = Scrubber()
        text = """
        AWS: AKIAIOSFODNN7EXAMPLE
        OpenAI: sk-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuv
        Random: 9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08
        """
        
        scrubbed, mappings = scrubber.scrub_prompt(text)
        
        assert "AKIAIOSFODNN7EXAMPLE" not in scrubbed
        assert "sk-ABCDEFGHIJKLMNOPQRSTUVWXYZ" not in scrubbed
        assert "9f86d081884c7d659a2feaa0c55ad015" not in scrubbed
        assert len(mappings) >= 3
    
    def test_rehydrate(self):
        scrubber = Scrubber()
        original = "My key is AKIAIOSFODNN7EXAMPLE"
        scrubbed, mappings = scrubber.scrub_prompt(original)
        rehydrated = scrubber.rehydrate(scrubbed, mappings)
        
        assert rehydrated == original
    
    def test_no_secrets(self):
        scrubber = Scrubber()
        text = "This is just normal text without secrets"
        scrubbed, mappings = scrubber.scrub_prompt(text)
        
        assert scrubbed == text
        assert len(mappings) == 0
    
    def test_overlapping_secrets(self):
        scrubber = Scrubber()
        text = "Key: AKIAIOSFODNN7EXAMPLEAKIAIOSFODNN7EXAMPLE"
        scrubbed, mappings = scrubber.scrub_prompt(text)
        
        assert "AKIAIOSFODNN7EXAMPLE" not in scrubbed
    
    def test_disable_detectors(self):
        scrubber = Scrubber(enable_regex=False, enable_entropy=True)
        text = "AWS: AKIAIOSFODNN7EXAMPLE"
        scrubbed, mappings = scrubber.scrub_prompt(text)
        
        assert scrubbed == text
    
    def test_custom_entropy_settings(self):
        scrubber = Scrubber(
            enable_regex=False,
            enable_entropy=True,
            min_entropy=2.0,
            min_entropy_length=10
        )
        text = "Short high entropy: Ab3$5^7*9!Qw2"
        matches = scrubber.scan(text)
        
        assert len(matches) > 0


class TestConvenienceFunctions:
    def test_scrub_prompt_function(self):
        text = "API key: sk-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuv"
        scrubbed, mappings = scrub_prompt(text)
        
        assert "sk-ABCDEFGHIJKLMNOPQRSTUVWXYZ" not in scrubbed
        assert len(mappings) == 1
    
    def test_scrub_response_function(self):
        text = "Generated: ghp_1234567890abcdefghijklmnopqrstuvwxyz"
        scrubbed = scrub_response(text)
        
        assert "ghp_" not in scrubbed or "REDACTED" in scrubbed


import contextvars
import uuid
from typing import Dict, Optional, Tuple
from threading import Lock


_placeholder_store: contextvars.ContextVar[Dict[str, str]] = contextvars.ContextVar(
    'placeholder_store', default={}
)
_global_lock = Lock()


class PlaceholderManager:
    def __init__(self, prefix: str = "SECRET", use_uuid: bool = False):
        self.prefix = prefix
        self.use_uuid = use_uuid
        self._counter = 0
        self._local_lock = Lock()
    
    def generate_placeholder(self, value: str) -> str:
        store = _placeholder_store.get()
        
        for placeholder, stored_value in store.items():
            if stored_value == value:
                return placeholder
        
        if self.use_uuid:
            placeholder = f"<{self.prefix}_{uuid.uuid4().hex[:8].upper()}>"
        else:
            with self._local_lock:
                self._counter += 1
                placeholder = f"<{self.prefix}_{self._counter}>"
        
        new_store = store.copy()
        new_store[placeholder] = value
        _placeholder_store.set(new_store)
        
        return placeholder
    
    def store_mapping(self, placeholder: str, value: str) -> None:
        store = _placeholder_store.get()
        new_store = store.copy()
        new_store[placeholder] = value
        _placeholder_store.set(new_store)
    
    def get_value(self, placeholder: str) -> Optional[str]:
        store = _placeholder_store.get()
        return store.get(placeholder)
    
    def get_all_mappings(self) -> Dict[str, str]:
        return _placeholder_store.get().copy()
    
    def clear(self) -> None:
        _placeholder_store.set({})
    
    def replace_secrets(self, text: str, secrets: list) -> Tuple[str, Dict[str, str]]:
        result = text
        mappings = {}
        
        sorted_secrets = sorted(secrets, key=lambda x: x.start, reverse=True)
        
        for secret in sorted_secrets:
            placeholder = self.generate_placeholder(secret.value)
            result = result[:secret.start] + placeholder + result[secret.end:]
            mappings[placeholder] = secret.value
        
        return result, mappings
    
    def rehydrate(self, text: str, mappings: Optional[Dict[str, str]] = None) -> str:
        if mappings is None:
            mappings = self.get_all_mappings()
        
        result = text
        for placeholder, value in mappings.items():
            result = result.replace(placeholder, value)
        
        return result


#!/usr/bin/env python3
import sys
import argparse
from pathlib import Path
from typing import List, Optional
from .scrubber import Scrubber


def scrub_file(file_path: Path, scrubber: Scrubber, output_path: Optional[Path] = None) -> bool:
    try:
        content = file_path.read_text()
        matches = scrubber.scan(content)
        
        if matches:
            print(f"Found {len(matches)} potential secrets in {file_path}:")
            for match in matches:
                print(f"  - {match.pattern_name}: {match.value[:20]}... (line {content[:match.start].count(chr(10)) + 1})")
            
            scrubbed_content = scrubber.scrub_response(content)
            
            if output_path:
                output_path.write_text(scrubbed_content)
                print(f"Scrubbed content written to {output_path}")
            else:
                print("\nScrubbed content:")
                print(scrubbed_content)
            
            return True
        else:
            print(f"No secrets found in {file_path}")
            return False
            
    except Exception as e:
        print(f"Error processing {file_path}: {e}", file=sys.stderr)
        return False


def scrub_stdin(scrubber: Scrubber) -> bool:
    content = sys.stdin.read()
    matches = scrubber.scan(content)
    
    if matches:
        print(f"Found {len(matches)} potential secrets in stdin:", file=sys.stderr)
        for match in matches:
            print(f"  - {match.pattern_name}: {match.value[:20]}...", file=sys.stderr)
        
        scrubbed_content = scrubber.scrub_response(content)
        print(scrubbed_content)
        return True
    else:
        print(content)
        return False


def main():
    parser = argparse.ArgumentParser(
        description="Scrub secrets from files or stdin",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  scrub-llm file.log                    # Check file for secrets
  scrub-llm file.log -o cleaned.log     # Write scrubbed output
  cat file.log | scrub-llm              # Process stdin
  scrub-llm *.log                       # Process multiple files
"""
    )
    
    parser.add_argument(
        'files',
        nargs='*',
        type=Path,
        help='Files to scrub (if none provided, reads from stdin)'
    )
    
    parser.add_argument(
        '-o', '--output',
        type=Path,
        help='Output file for scrubbed content (only valid with single input file)'
    )
    
    parser.add_argument(
        '--no-regex',
        action='store_true',
        help='Disable regex-based detection'
    )
    
    parser.add_argument(
        '--no-entropy',
        action='store_true',
        help='Disable entropy-based detection'
    )
    
    parser.add_argument(
        '--min-entropy',
        type=float,
        default=3.5,
        help='Minimum Shannon entropy for detection (default: 3.5)'
    )
    
    parser.add_argument(
        '--min-length',
        type=int,
        default=20,
        help='Minimum string length for entropy detection (default: 20)'
    )
    
    args = parser.parse_args()
    
    if args.output and len(args.files) != 1:
        parser.error("--output can only be used with a single input file")
    
    scrubber = Scrubber(
        enable_regex=not args.no_regex,
        enable_entropy=not args.no_entropy,
        min_entropy=args.min_entropy,
        min_entropy_length=args.min_length
    )
    
    found_secrets = False
    
    if not args.files:
        found_secrets = scrub_stdin(scrubber)
    else:
        for file_path in args.files:
            if not file_path.exists():
                print(f"Error: {file_path} does not exist", file=sys.stderr)
                continue
            
            output_path = args.output if len(args.files) == 1 else None
            if scrub_file(file_path, scrubber, output_path):
                found_secrets = True
    
    sys.exit(1 if found_secrets else 0)


if __name__ == '__main__':
    main()


import pytest
from scrub_llm.detectors import RegexDetector, EntropyDetector


class TestRegexDetector:
    def test_aws_keys(self):
        detector = RegexDetector()
        text = "My AWS key is AKIAIOSFODNN7EXAMPLE and secret is wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY"
        matches = detector.scan(text)
        
        assert len(matches) >= 1
        assert any(m.pattern_name == "aws_access_key_id" for m in matches)
        assert any(m.value == "AKIAIOSFODNN7EXAMPLE" for m in matches)
    
    def test_github_token(self):
        detector = RegexDetector()
        text = "Use this token: ghp_1234567890abcdefghijklmnopqrstuvwxyz"
        matches = detector.scan(text)
        
        assert len(matches) == 1
        assert matches[0].pattern_name == "github_token"
        assert matches[0].value == "ghp_1234567890abcdefghijklmnopqrstuvwxyz"
    
    def test_openai_key(self):
        detector = RegexDetector()
        text = "OPENAI_API_KEY=sk-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuv"
        matches = detector.scan(text)
        
        assert len(matches) == 1
        assert matches[0].pattern_name == "openai_api_key"
    
    def test_multiple_secrets(self):
        detector = RegexDetector()
        text = """
        AWS: AKIAIOSFODNN7EXAMPLE
        GitHub: ghp_1234567890abcdefghijklmnopqrstuvwxyz
        Slack: xoxb-1234567890-abcdefghijk
        """
        matches = detector.scan(text)
        
        assert len(matches) >= 3
        pattern_names = {m.pattern_name for m in matches}
        assert "aws_access_key_id" in pattern_names
        assert "github_token" in pattern_names
        assert "slack_token" in pattern_names
    
    def test_no_secrets(self):
        detector = RegexDetector()
        text = "This is just a normal text without any secrets."
        matches = detector.scan(text)
        
        assert len(matches) == 0


class TestEntropyDetector:
    def test_high_entropy_string(self):
        detector = EntropyDetector(min_length=20, min_entropy=3.5)
        text = "Random string: 9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08"
        matches = detector.scan(text)
        
        assert len(matches) == 1
        assert matches[0].pattern_name == "high_entropy_string"
        assert matches[0].metadata["entropy"] > 3.5
    
    def test_low_entropy_string(self):
        detector = EntropyDetector(min_length=20, min_entropy=3.5)
        text = "Normal text: aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
        matches = detector.scan(text)
        
        assert len(matches) == 0
    
    def test_short_string_ignored(self):
        detector = EntropyDetector(min_length=20, min_entropy=3.5)
        text = "Short: Ab3$5^7*9"
        matches = detector.scan(text)
        
        assert len(matches) == 0
    
    def test_entropy_calculation(self):
        detector = EntropyDetector()
        
        entropy_low = detector._calculate_shannon_entropy("aaaaaaaaaa")
        entropy_med = detector._calculate_shannon_entropy("abcabcabca")
        entropy_high = detector._calculate_shannon_entropy("9f86d081884c7d659a2f")
        
        assert entropy_low < entropy_med < entropy_high
        assert entropy_low < 1.0
        assert entropy_high > 3.0